@@ -24,8 +24,8 @@
 // local
 #include "categories.h"
 #include "collectors.h"
+#include "locationaccesshelper.h"
 #include "previewmodel.h"
-#include "locationservice.h"
 #include "utils.h"
 #include "scopes.h"
 #include "settingsmodel.h"
@@ -75,12 +75,13 @@ const int RESULTS_TTL_MEDIUM = 300000; // 5 minutes
 const int RESULTS_TTL_LARGE = 3600000; // 1 hour
 const int SEARCH_CARDINALITY = 300; // maximum number of results accepted from a single scope
 
-Scope::Ptr Scope::newInstance(scopes_ng::Scopes* parent)
+Scope::Ptr Scope::newInstance(scopes_ng::Scopes* parent, bool favorite)
 {
-    return Scope::Ptr(new Scope(parent), &QObject::deleteLater);
+    auto scope = Scope::Ptr(new Scope(parent, favorite), &QObject::deleteLater);
+    return scope;
 }
 
-Scope::Scope(scopes_ng::Scopes* parent) :
+Scope::Scope(scopes_ng::Scopes* parent, bool favorite) :
       m_query_id(0)
     , m_formFactor(QStringLiteral("phone"))
     , m_activeFiltersCount(0)
@@ -90,7 +91,7 @@ Scope::Scope(scopes_ng::Scopes* parent) :
     , m_resultsDirty(false)
     , m_delayedSearchProcessing(false)
     , m_hasNavigation(false)
-    , m_favorite(false)
+    , m_favorite(favorite)
     , m_initialQueryDone(false)
     , m_childScopesDirty(true)
     , m_searchController(new CollectionController)
@@ -127,12 +128,11 @@ Scope::Scope(scopes_ng::Scopes* parent) :
     QObject::connect(&m_searchProcessingDelayTimer, SIGNAL(timeout()), this, SLOT(flushUpdates()));
     m_invalidateTimer.setSingleShot(true);
     m_invalidateTimer.setTimerType(Qt::CoarseTimer);
-    QObject::connect(&m_invalidateTimer, &QTimer::timeout, this, &Scope::invalidateResults);
+    QObject::connect(&m_invalidateTimer, &QTimer::timeout, [this]() { invalidateResults(); });
 }
 
 Scope::~Scope()
 {
-    m_childScopesFuture.waitForFinished();
 }
 
 void Scope::processSearchChunk(PushEvent* pushEvent)
@@ -373,7 +373,7 @@ void Scope::flushUpdates(bool finalize)
     }
 
 #ifdef VERBOSE_MODEL_UPDATES
-    qDebug() << "flushUpdates:" << id() << "#results =" << m_cachedResults.count() << "finalize:" << finalize;
+    qDebug() << id() << ": flushUpdates:" << "#results =" << m_cachedResults.count() << "finalize:" << finalize;
 #endif
 
     processResultSet(m_cachedResults); // clears the result list
@@ -432,7 +432,7 @@ void Scope::flushUpdates(bool finalize)
         }
 
         if (!containsDepartments && !m_currentNavigationId.isEmpty()) {
-            qDebug() << "Resetting current nav id";
+            qDebug() << id() << ": Resetting current nav id";
             m_currentNavigationId = QLatin1String("");
             Q_EMIT currentNavigationIdChanged();
         }
@@ -442,7 +442,7 @@ void Scope::flushUpdates(bool finalize)
     // process filters
     if (finalize || m_receivedFilters.size() > 0)
     {
-        qDebug() << "Processing" << m_receivedFilters.size() << "filters";
+        qDebug() << id() << ": Processing" << m_receivedFilters.size() << "filters";
         const bool containsFilters = (m_receivedFilters.size() > 0);
         const bool haveFiltersAlready = (m_filters->rowCount() > 0);
         if (containsFilters) {
@@ -451,11 +451,11 @@ void Scope::flushUpdates(bool finalize)
             if (!haveFiltersAlready) {
                 Q_EMIT filtersChanged();
             }
-            qDebug() << "Current number of filters:" << m_filters->rowCount();
+            qDebug() << id() << ": Current number of filters:" << m_filters->rowCount();
         }
         else
         {
-            qDebug() << "Removing all filters";
+            qDebug() << id() << ": Removing all filters";
             m_filters->clear();
             if (haveFiltersAlready) {
                 Q_EMIT filtersChanged();
@@ -684,7 +684,7 @@ void Scope::setStatus(shell::scopes::ScopeInterface::Status status)
 void Scope::setCurrentNavigationId(QString const& id)
 {
     if (m_currentNavigationId != id) {
-        qDebug() << "Setting current nav id:" <<  this->id() << id;
+        qDebug() << this->id() << ": Setting current nav id:" <<  this->id() << id;
         processPrimaryNavigationTag(id);
         m_currentNavigationId = id;
         Q_EMIT currentNavigationIdChanged();
@@ -696,7 +696,7 @@ void Scope::setFilterState(scopes::FilterState const& filterState)
     m_filterState = filterState;
 }
 
-void Scope::dispatchSearch()
+void Scope::dispatchSearch(bool programmaticSearch)
 {
     m_initialQueryDone = true;
 
@@ -736,6 +736,20 @@ void Scope::dispatchSearch()
     // (i.e. while the loading bar is visible).
     update_child_scopes();
 
+    // handle the case where single scope is refreshed multiple times without switching
+    // to another scope, which would never trigger location prompt.
+    if (m_scopeMetadata && m_scopeMetadata->location_data_needed() && !m_locationToken)
+    {
+        if (m_isActive)
+        {
+            Q_ASSERT(m_scopesInstance);
+
+            if ((!programmaticSearch) || m_scopesInstance->locationAccessHelper()->trustedPromptWasShown()) {
+                m_locationToken = m_locationService->activate();
+            }
+        }
+    }
+
     if (m_proxy) {
         scopes::SearchMetadata meta(m_cardinality, QLocale::system().name().toStdString(), m_formFactor.toStdString());
         auto const userAgent = m_scopesInstance->userAgentString();
@@ -764,8 +778,9 @@ void Scope::dispatchSearch()
 
         scopes::SearchListenerBase::SPtr listener(new SearchResultReceiver(this));
         m_searchController->setListener(listener);
+
         try {
-            qDebug() << "Dispatching search:" << id() << m_searchQuery << m_currentNavigationId;
+            qDebug() << id() << ": Dispatching search:" << m_searchQuery << m_currentNavigationId << "(programmatic:" << programmaticSearch << ")";
             scopes::QueryCtrlProxy controller = m_queryUserData ?
                 m_proxy->search(m_searchQuery.toStdString(), m_currentNavigationId.toStdString(), m_filterState, *m_queryUserData, meta, listener) :
                 m_proxy->search(m_searchQuery.toStdString(), m_currentNavigationId.toStdString(), m_filterState, meta, listener);
@@ -792,6 +807,11 @@ void Scope::setScopeData(scopes::ScopeMetadata const& data)
     m_customizations = converted.toMap();
     Q_EMIT customizationsChanged();
 
+    createSettingsModel();
+}
+
+void Scope::createSettingsModel()
+{
     try
     {
         scopes::Variant settings_definitions;
@@ -806,10 +826,20 @@ void Scope::setScopeData(scopes::ScopeMetadata const& data)
             shareDir = QDir::home().filePath(QStringLiteral(".config/unity-scopes"));
         }
 
+        Q_ASSERT(m_scopesInstance);
+
         m_settingsModel.reset(
                 new SettingsModel(shareDir, id(),
-                        scopeVariantToQVariant(settings_definitions), this));
-        QObject::connect(m_settingsModel.data(), &SettingsModel::settingsChanged, this, &Scope::invalidateResults);
+                        scopeVariantToQVariant(settings_definitions),
+                        !m_scopesInstance->locationAccessHelper()->isLocationAccessDenied(),
+                        this));
+
+        QObject::connect(m_settingsModel.data(), &SettingsModel::settingsChanged, [this]() { invalidateResults(); });
+
+        // If the scope needs location, then changes to global location access need to be monitored.
+        if (m_scopeMetadata->location_data_needed()) {
+            QObject::connect(m_scopesInstance->locationAccessHelper().data(), &LocationAccessHelper::accessChanged, this, &Scope::locationAccessChanged);
+        }
     }
     catch (unity::scopes::NotFoundException&)
     {
@@ -906,6 +936,16 @@ unity::shell::scopes::SettingsModelInterface* Scope::settings() const
     return m_settingsModel.data();
 }
 
+void Scope::locationAccessChanged()
+{
+    qDebug() << id() << ": Location access changed, recreating settings model";
+    createSettingsModel();
+
+    // Force child scopes refresh
+    m_childScopesDirty = true;
+    update_child_scopes();
+}
+
 bool Scope::require_child_scopes_refresh() const
 {
     if (m_settingsModel && m_scopesInstance)
@@ -923,14 +963,7 @@ void Scope::update_child_scopes()
         // reset the flag so that re-entering this method won't restart the update unnecessarily
         m_childScopesDirty = false;
 
-        // just in case we have another update still running, wait here for it to complete
-        m_childScopesFuture.waitForFinished();
-
-        // run the update in a seperate thread
-        m_childScopesFuture = QtConcurrent::run([this]
-        {
-            m_settingsModel->update_child_scopes(m_scopesInstance->getAllMetadata());
-        });
+        m_settingsModel->update_child_scopes(m_scopesInstance->getAllMetadata());
     }
 }
 
@@ -985,7 +1018,7 @@ void Scope::previewModelDestroyed(QObject *obj)
     for (auto it = m_previewModels.begin(); it != m_previewModels.end(); it++)
     {
         if (*it == obj) {
-            qDebug() << "PreviewModel destroyed";
+            qDebug() << id() << ": PreviewModel destroyed";
             m_previewModels.erase(it);
             break;
         }
@@ -1110,17 +1143,15 @@ void Scope::setFormFactor(const QString& form_factor) {
 }
 
 void Scope::setActive(const bool active) {
+    qDebug() << id() << ": setActive:" << active;
+
     if (active != m_isActive) {
         m_isActive = active;
         Q_EMIT isActiveChanged();
 
         if (m_scopeMetadata && m_scopeMetadata->location_data_needed())
         {
-            if (m_isActive)
-            {
-                m_locationToken = m_locationService->activate();
-            }
-            else
+            if (!m_isActive)
             {
                 m_locationToken.reset();
             }
@@ -1157,7 +1188,7 @@ void Scope::activate(QVariant const& result_var, QString const& categoryId)
 
     auto const activateResult = [this, result, categoryId, result_var]() {
         if (result->direct_activation()) {
-            if (result->uri().find("scope://") == 0 || id() == QLatin1String("clickscope") || (id() == QLatin1String("videoaggregator") && categoryId == QLatin1String("myvideos-getstarted"))) {
+            if (result->uri().find("scope://") == 0 || id() == QLatin1String("clickscope") || id() == QLatin1String("libertine-scope.ubuntu_libertine-scope") || (id() == QLatin1String("videoaggregator") && categoryId == QLatin1String("myvideos-getstarted"))) {
                 activateUri(QString::fromStdString(result->uri()));
             } else {
                 Q_EMIT previewRequested(result_var);
@@ -1237,7 +1268,7 @@ void Scope::activateAction(QVariant const& result_var, QString const& categoryId
         scopes::ActivationListenerBase::SPtr listener(new ActivationReceiver(this, result, categoryId));
         m_activationController->setListener(listener);
 
-        qDebug() << "Activating result action for result with uri '" << QString::fromStdString(result->uri()) << ", categoryId" << categoryId;
+        qDebug() << id() << ": Activating result action for result with uri '" << QString::fromStdString(result->uri()) << ", categoryId" << categoryId;
 
         auto proxy = proxy_for_result(result);
         unity::scopes::ActionMetadata metadata(QLocale::system().name().toStdString(), m_formFactor.toStdString());
@@ -1286,10 +1317,17 @@ void Scope::invalidateChildScopes()
     m_childScopesDirty = true;
 }
 
-void Scope::invalidateResults()
+void Scope::invalidateResults(bool programmaticSearch)
 {
-    if (m_isActive) {
-        dispatchSearch();
+    qDebug() << id() << ": results invalidated, programmatic:" << programmaticSearch << ", active:" << m_isActive;
+
+    // If it's a programmatic invalidation (due to scope registry changes for example) and the trusted prompt was never shown before,
+    // then dispatch search even if scope is not currently active; just setting 'dirty' flag is problematic because when the scope
+    // becomes active we won't know if it was because of programmatic search.
+    bool firstBoot = (m_scopesInstance != nullptr && !m_scopesInstance->locationAccessHelper()->trustedPromptWasShown());
+
+    if (m_isActive || (programmaticSearch && firstBoot)) {
+        dispatchSearch(programmaticSearch);
     } else {
         // mark the results as dirty, so next setActive() re-sends the query
         if (!m_resultsDirty)
@@ -1302,7 +1340,7 @@ void Scope::invalidateResults()
 
 void Scope::resetPrimaryNavigationTag()
 {
-    qDebug() << "resetPrimaryNavigationTag()";
+    qDebug() << id() << ": resetPrimaryNavigationTag()";
     setCurrentNavigationId("");
     m_filters->update(unity::scopes::FilterState());
     filterStateChanged();
@@ -1313,7 +1351,7 @@ void Scope::resetPrimaryNavigationTag()
     if (m_typingTimer.isActive()) {
         m_typingTimer.stop();
         Q_EMIT searchQueryChanged();
-    }  
+    }
 }
 
 void Scope::resetFilters()
@@ -1385,7 +1423,7 @@ QString Scope::primaryNavigationTag() const
 
 void Scope::filterStateChanged()
 {
-    qDebug() << "Filters changed";
+    qDebug() << id() << ": Filters changed";
     m_filterState = m_filters->filterState();
     processPrimaryNavigationTag(m_currentNavigationId);
     processActiveFiltersCount();
@@ -1401,7 +1439,7 @@ void Scope::processActiveFiltersCount()
         m_activeFiltersCount = count;
         Q_EMIT activeFiltersCountChanged();
     }
-    qDebug() << "active filters count:" << m_activeFiltersCount;
+    qDebug() << id() << ": Active filters count:" << m_activeFiltersCount;
 }
 
 //
@@ -1436,7 +1474,7 @@ void Scope::processPrimaryNavigationTag(QString const &targetDepartmentId)
             tag = pf->filterTag();
         }
     }
-    qDebug() << "Scope::processPrimaryNavigationTag(): tag is '" << tag << "'";
+    qDebug() << id() << ": processPrimaryNavigationTag(): tag is '" << tag << "'";
     if (m_primaryNavigationTag != tag) {
         m_primaryNavigationTag = tag;
         Q_EMIT primaryNavigationTagChanged();
