@@ -74,7 +74,8 @@ static const char* GROUP_NAME = "General";
 }  // namespace
 
 SettingsModel::SettingsModel(const QDir& configDir, const QString& scopeId,
-        const QVariant& settingsDefinitions, QObject* parent,
+        const QVariant& settingsDefinitions, bool isLocationGloballyEnabled,
+        QObject* parent,
         int settingsTimeout)
         : SettingsModelInterface(parent), m_scopeId(scopeId), m_settingsTimeout(settingsTimeout),
           m_requireChildScopesRefresh(false)
@@ -86,12 +87,11 @@ SettingsModel::SettingsModel(const QDir& configDir, const QString& scopeId,
 
     try
     {
-        tryLoadSettings();
+        tryLoadSettings(true);
     }
-    catch(const unity::FileException& e)
+    catch(const unity::FileException&)
     {
-        // Something has gone wrong, at this point we'll just have to continue with a null m_settings.
-        qWarning() << "SettingsModel::SettingsModel: Failed to read settings file:" << e.what();
+        // No settings file found, at this point we'll just have to continue with a null m_settings.
     }
 
     for (const auto &it : settingsDefinitions.toList())
@@ -99,6 +99,12 @@ SettingsModel::SettingsModel(const QDir& configDir, const QString& scopeId,
         QVariantMap data = it.toMap();
         QString id = data[QStringLiteral("id")].toString();
         QString displayName = data[QStringLiteral("displayName")].toString();
+
+        if (id == "internal.location" && !isLocationGloballyEnabled) {
+            qDebug() << "Location setting ignored, waiting for global location access to be enabled first";
+            continue;
+        }
+
         QVariantMap properties;
         QVariant defaultValue;
         if (data.contains(QStringLiteral("displayValues")))
@@ -152,8 +158,6 @@ SettingsModel::SettingsModel(const QDir& configDir, const QString& scopeId,
 
 QVariant SettingsModel::data(const QModelIndex& index, int role) const
 {
-    QMutexLocker locker(&m_mutex);
-
     int row = index.row();
     QVariant result;
 
@@ -179,7 +183,7 @@ QVariant SettingsModel::data(const QModelIndex& index, int role) const
             {
                 try
                 {
-                    tryLoadSettings();
+                    tryLoadSettings(true);
                     switch (data->variantType)
                     {
                         case QVariant::Bool:
@@ -249,8 +253,6 @@ QVariant SettingsModel::data(const QModelIndex& index, int role) const
 
 QVariant SettingsModel::value(const QString& id) const
 {
-    QMutexLocker locker(&m_mutex);
-
     // Check for the setting id in the child scopes list first, in case the
     // aggregator is incorrectly using a scope id as a settings as well.
     if (m_child_scopes_data_by_id.contains(id))
@@ -269,7 +271,7 @@ QVariant SettingsModel::value(const QString& id) const
         QVariant result;
         try
         {
-            tryLoadSettings();
+            tryLoadSettings(true);
             switch (data->variantType)
             {
                 case QVariant::Bool:
@@ -307,8 +309,6 @@ QVariant SettingsModel::value(const QString& id) const
 
 void SettingsModel::update_child_scopes(QMap<QString, sc::ScopeMetadata::SPtr> const& scopes_metadata)
 {
-    QMutexLocker locker(&m_mutex);
-
     if (!scopes_metadata.contains(m_scopeId) ||
         !scopes_metadata[m_scopeId]->is_aggregator())
     {
@@ -363,17 +363,13 @@ void SettingsModel::update_child_scopes(QMap<QString, sc::ScopeMetadata::SPtr> c
         endResetModel();
     }
 
-    locker.unlock();
     Q_EMIT countChanged();
 }
 
 bool SettingsModel::setData(const QModelIndex &index, const QVariant &value,
         int role)
 {
-    QMutexLocker locker(&m_mutex);
-
     int row = index.row();
-    QVariant result;
 
     if (row < m_data.size())
     {
@@ -435,20 +431,16 @@ int SettingsModel::rowCount(const QModelIndex&) const
 
 int SettingsModel::count() const
 {
-    QMutexLocker locker(&m_mutex);
     return m_data.size() + m_child_scopes_data.size();
 }
 
 bool SettingsModel::require_child_scopes_refresh() const
 {
-    QMutexLocker locker(&m_mutex);
     return m_requireChildScopesRefresh;
 }
 
 void SettingsModel::settings_timeout()
 {
-    QMutexLocker locker(&m_mutex);
-
     QObject *timer = sender();
     if (!timer)
     {
@@ -472,7 +464,6 @@ void SettingsModel::settings_timeout()
             {
                 m_scopeProxy->set_child_scopes(m_child_scopes);
 
-                locker.unlock();
                 Q_EMIT settingsChanged();
             }
             catch (std::exception const& e)
@@ -486,7 +477,7 @@ void SettingsModel::settings_timeout()
     {
         try
         {
-            tryLoadSettings();
+            tryLoadSettings(false);
             switch (value.type())
             {
                 case QVariant::Bool:
@@ -508,7 +499,6 @@ void SettingsModel::settings_timeout()
             FileLock lock = unixLock(m_settings_path, true);
             m_settings->sync(); // make sure the change to setting value is synced to fs
 
-            locker.unlock();
             Q_EMIT settingsChanged();
         }
         catch(const unity::FileException& e)
@@ -526,15 +516,19 @@ void SettingsModel::settings_timeout()
     }
 }
 
-void SettingsModel::tryLoadSettings() const
+void SettingsModel::tryLoadSettings(bool read_only) const
 {
     if (!m_settings)
     {
         QFileInfo checkFile(m_settings_path);
         if (!checkFile.exists() || !checkFile.isFile())
         {
+            if (read_only)
+            {
+                throw unity::FileException("Could not locate a settings file at: " + m_settings_path.toStdString(), -1);
+            }
             // Config file does not exist, so we create an empty one.
-            if (!QFile(m_settings_path).open(QFile::WriteOnly))
+            else if (!QFile(m_settings_path).open(QFile::WriteOnly))
             {
                 throw unity::FileException("Could not create an empty settings file at: " + m_settings_path.toStdString(), -1);
             }
