@@ -129,7 +129,7 @@ public:
             reply->push(res);
         }
         else if (query_ == "layout" || query_ == "layout-push-order-change" || query_ == "layout-order-change" || query_ == "incomplete-layout"
-                 || query_ == "preview-replace-with-removal" || query_ == "preview-replace-with-moves")
+                 || query_ == "preview-replace-with-removal" || query_ == "preview-replace-with-moves" || query_ == "preview-replace-no-layouts")
         {
             CategoryRenderer minimal_rndr(R"({"schema-version": 1, "components": {"title": "title"}})");
             auto cat = reply->register_category("cat1", "Category 1", "", minimal_rndr);
@@ -288,6 +288,15 @@ public:
             res["setting-distanceUnit"] = settings_["distanceUnit"];
             reply->push(res);
         }
+        else if (query_ == "preview-with-duplicated-widget-id")
+        {
+            auto cat = reply->register_category("cat1", "Category 1", "");
+            CategorisedResult res(cat);
+            res.set_uri(query_);
+            res.set_title("result for: \"" + query_ + "\"");
+            res.set_art("art");
+            reply->push(res);
+        }
         else
         {
             auto cat = reply->register_category("cat1", "Category 1", "");
@@ -360,6 +369,15 @@ public:
             reply->push(widgets);
             return;
         }
+        else if (result().uri().find("preview-with-duplicated-widget-id") != std::string::npos)
+        {
+            PreviewWidget w1("img", "image");
+            PreviewWidget w2("author", "header");
+            PreviewWidget w3("hdr", "header");
+            PreviewWidget w4("author", "text");
+
+            reply->push({w1, w2, w3, w4});
+        }
         else if (result().uri().find("preview-replace-with-removal") != std::string::npos)
         {
             PreviewWidget w1("img", "image");
@@ -389,6 +407,22 @@ public:
             reply->push(widgets);
             return;
         }
+        else if (result().uri().find("preview-replace-no-layouts") != std::string::npos)
+        {
+            PreviewWidget w1("img", "image");
+            PreviewWidget w2("hdr", "header");
+            PreviewWidget w3("desc", "text");
+            PreviewWidget w4("actions", "actions");
+
+            VariantBuilder builder;
+            builder.add_tuple({
+                {"id", Variant("download")},
+                {"label", Variant("Download")}
+            });
+            w4.add_attribute_value("actions", builder.end());
+            reply->push({w1, w2, w3, w4});
+            return;
+        }
         else if (result().uri().find("preview-replace-with-moves") != std::string::npos)
         {
             PreviewWidget w1("img", "image");
@@ -722,7 +756,7 @@ public:
         return query;
     }
 
-    virtual ActivationQueryBase::UPtr perform_action(Result const& result, ActionMetadata const& meta, std::string const& widget_id, std::string const& action_id)
+    virtual ActivationQueryBase::UPtr perform_action(Result const& result, ActionMetadata const& meta, std::string const& widget_id, std::string const& action_id) override
     {
         cout << "scope-A: called perform_action: " << widget_id << ", " << action_id << endl;
         if (widget_id == "actions" && action_id == "hide")
