@@ -135,6 +135,48 @@ private Q_SLOTS:
         );
     }
 
+    void testPreviewWithDuplicatedWidgetId()
+    {
+        m_resultsView->setQuery("preview-with-duplicated-widget-id");
+
+        QVERIFY_MATCHRESULT(
+            shm::CategoryListMatcher()
+                .hasAtLeast(1)
+                .mode(shm::CategoryListMatcher::Mode::starts_with)
+                .category(shm::CategoryMatcher("cat1")
+                    .hasAtLeast(1)
+                    .mode(shm::CategoryMatcher::Mode::starts_with)
+                    .result(shm::ResultMatcher("preview-with-duplicated-widget-id"))
+                )
+                .match(m_resultsView->categories())
+        );
+
+        auto abstractView = m_resultsView->category(0).result(0).longPress();
+        QVERIFY(bool(abstractView));
+        auto previewView = dynamic_pointer_cast<shv::PreviewView>(abstractView);
+        QVERIFY(bool(previewView));
+
+        QVERIFY_MATCHRESULT(
+            shm::PreviewColumnMatcher()
+            .column(
+                shm::PreviewMatcher()
+                .widget(
+                    shm::PreviewWidgetMatcher("img")
+                    .type("image")
+                )
+                .widget(
+                    shm::PreviewWidgetMatcher("author")
+                    .type("header")
+                )
+                .widget(
+                    shm::PreviewWidgetMatcher("hdr")
+                    .type("header")
+                )
+            )
+            .match(previewView->widgets())
+        );
+    }
+
     void testExpandablePreviewWidget()
     {
 //        QScopedPointer<PreviewStack> preview_stack;
@@ -609,6 +651,46 @@ private Q_SLOTS:
             .match(previewView2->widgets())
             );
     }
+
+    void testPreviewReplacingPreviewWithoutLayouts()
+    {
+        m_resultsView->setQuery("preview-replace-no-layouts");
+
+        auto abstractView = m_resultsView->category(0).result(0).longPress();
+        QVERIFY(bool(abstractView));
+        auto previewView = dynamic_pointer_cast<shv::PreviewView>(abstractView);
+        QVERIFY(bool(previewView));
+
+        QVERIFY_MATCHRESULT(
+            shm::PreviewColumnMatcher()
+            .column(
+                shm::PreviewMatcher()
+                .widget(shm::PreviewWidgetMatcher("img"))
+                .widget(shm::PreviewWidgetMatcher("hdr"))
+                .widget(shm::PreviewWidgetMatcher("desc"))
+                .widget(shm::PreviewWidgetMatcher("actions"))
+            )
+            .match(previewView->widgets())
+        );
+
+        sc::VariantMap hints {{"session-id", sc::Variant("goo")}};
+        auto sameView = previewView->widgetsInFirstColumn().at("actions").trigger("download", sc::Variant(hints));
+        auto previewView2 = dynamic_pointer_cast<shv::PreviewView>(sameView);
+        QVERIFY(bool(previewView2));
+
+        // the order of widgets shouldn't change
+        QVERIFY_MATCHRESULT(
+            shm::PreviewColumnMatcher()
+            .column(
+                shm::PreviewMatcher()
+                .widget(shm::PreviewWidgetMatcher("img"))
+                .widget(shm::PreviewWidgetMatcher("hdr"))
+                .widget(shm::PreviewWidgetMatcher("desc"))
+                .widget(shm::PreviewWidgetMatcher("actions"))
+            )
+            .match(previewView2->widgets())
+        );
+    }
 };
 
 QTEST_GUILESS_MAIN(PreviewTest)
